/*
 * Copyright (c) 2011 Peter Troshin JAva Bioinformatics Analysis Web Services
 * (JABAWS) @version: 2.0 This library is free software; you can redistribute it
 * and/or modify it under the terms of the Apache License version 2 as published
 * by the Apache Software Foundation This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Apache License for more details. A copy of the license is in
 * apache_license.txt. It is also available here:
 * @see: http://www.apache.org/licenses/LICENSE-2.0.txt Any republication or
 * derived work distributed in source code form must include this copyright and
 * license notice.
 */

package compbio.runner.disorder;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import compbio.data.sequence.Score;
import compbio.data.sequence.ScoreManager;
import compbio.data.sequence.SequenceUtil;
import compbio.data.sequence.UnknownFileFormatException;
import compbio.engine.client.SkeletalExecutable;
import compbio.metadata.ResultNotAvailableException;

/**
 * iupred sequenceFile <short long glob >
 * 
 * Maximum sequence length is 40000 chars. Single string length max is a 1000
 * chars!
 * 
 */
public class IUPred extends SkeletalExecutable<IUPred> {

	private static Logger log = Logger.getLogger(IUPred.class);
	private static final String GLOB_SUFFIX = ".glob";
	private static final String SHORT_SUFFIX = ".short";
	private static final String LONG_SUFFIX = ".long";

	@Override
	@SuppressWarnings("unchecked")
	public ScoreManager getResults(String workDirectory)
			throws ResultNotAvailableException {

		ScoreManager results = null;
		try {

			Map<String, Score> globScores = SequenceUtil.readIUPred(new File(
					workDirectory, getOutput() + GLOB_SUFFIX));
			Map<String, Score> shortScores = SequenceUtil.readIUPred(new File(
					workDirectory, getOutput() + SHORT_SUFFIX));
			Map<String, Score> longScores = SequenceUtil.readIUPred(new File(
					workDirectory, getOutput() + LONG_SUFFIX));
			Map<String, Set<Score>> combined = new TreeMap<String, Set<Score>>();
			for (String key : globScores.keySet()) {
				Set<Score> all = new TreeSet<Score>();
				Score globsc = globScores.get(key);
				assert globsc != null;
				Score shortsc = shortScores.get(key);
				assert shortsc != null;
				all.add(shortsc);
				Score longsc = longScores.get(key);
				assert longsc != null;
				all.add(longsc);
				combined.put(key, all);
			}
			results = ScoreManager.newInstance(combined);

		} catch (FileNotFoundException e) {
			log.error(e.getMessage(), e.getCause());
			throw new ResultNotAvailableException(e);
		} catch (IOException e) {
			log.error(e.getMessage(), e.getCause());
			throw new ResultNotAvailableException(e);
		} catch (UnknownFileFormatException e) {
			log.error(e.getMessage(), e.getCause());
			throw new ResultNotAvailableException(e);
		} catch (NullPointerException e) {
			log.error(e.getMessage(), e.getCause());
			throw new ResultNotAvailableException(e);
		}
		return results;
	}

	@Override
	public IUPred setInput(String inFile) {
		super.setInput(inFile);
		cbuilder.setFirst(inFile);
		return this;
	}

	@SuppressWarnings("unchecked")
	@Override
	public Class<IUPred> getType() {
		return (Class<IUPred>) this.getClass();
	}

}
